package com.chelvc.cloud.vehicle.server.service.impl;

import java.util.List;
import java.util.Objects;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.api.constant.CategoryType;
import com.chelvc.cloud.vehicle.api.constant.MerchantStatus;
import com.chelvc.cloud.vehicle.api.dto.GoodsDTO;
import com.chelvc.cloud.vehicle.api.dto.MerchantDTO;
import com.chelvc.cloud.vehicle.api.dto.MerchantDetailDTO;
import com.chelvc.cloud.vehicle.api.param.GoodsQueryParam;
import com.chelvc.cloud.vehicle.api.param.LocationQueryParam;
import com.chelvc.cloud.vehicle.api.param.MerchantModifyParam;
import com.chelvc.cloud.vehicle.api.param.MerchantPagingParam;
import com.chelvc.cloud.vehicle.api.param.MerchantQueryParam;
import com.chelvc.cloud.vehicle.server.copier.MerchantCopier;
import com.chelvc.cloud.vehicle.server.dao.MerchantMapper;
import com.chelvc.cloud.vehicle.server.entity.Merchant;
import com.chelvc.cloud.vehicle.server.service.GoodsService;
import com.chelvc.cloud.vehicle.server.service.MerchantService;
import com.chelvc.framework.base.util.ResourceUtils;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.util.PagingUtils;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 商家业务操作实现
 *
 * @author Woody
 * @date 2023/5/2
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@DubboService(interfaceClass = com.chelvc.cloud.vehicle.api.service.MerchantService.class)
public class MerchantServiceImpl extends ServiceImpl<MerchantMapper, Merchant> implements MerchantService,
        com.chelvc.cloud.vehicle.api.service.MerchantService {
    private final GoodsService goodsService;

    @Override
    public List<MerchantDTO> listNearbyMerchants(@NonNull LocationQueryParam param, int size) {
        return this.baseMapper.listNearbyMerchants(param, size);
    }

    @Override
    public List<MerchantDTO> listRecommendMerchants(@NonNull LocationQueryParam param, int size) {
        return this.baseMapper.listRecommendMerchants(param, size);
    }

    @Override
    public List<MerchantDTO> listSimpleMerchants(@NonNull MerchantQueryParam param) {
        return this.baseMapper.listSimpleMerchants(param);
    }

    @Override
    public MerchantDetailDTO getMerchantDetail(@NonNull Long id) {
        // 获取商家信息
        MerchantDTO merchant = MerchantCopier.INSTANCE.copying(
                ResourceUtils.required(this.getById(id), "商家不存在")
        );

        // 获取商家全部商品列表
        List<GoodsDTO> goods = this.goodsService.listMerchantSimpleGoods(
                id, GoodsQueryParam.builder().size(10).build()
        );

        // 获取商家商品分类类型列表
        List<CategoryType> types = this.goodsService.listMerchantGoodsCategoryTypes(id);
        return MerchantDetailDTO.builder().merchant(merchant).goods(goods).categories(types).build();
    }


    @Override
    public Long addMerchant(@NonNull MerchantModifyParam param) {
        Merchant merchant = MerchantCopier.INSTANCE.copying(param);
        merchant.setUserId(0L);
        merchant.setScore(0D);
        merchant.setSale(0);
        merchant.setStatus(MerchantStatus.ONLINE);
        this.save(merchant);
        return merchant.getId();
    }

    @Override
    public void updateMerchant(@NonNull Long id, @NonNull MerchantModifyParam param) {
        Merchant merchant = ResourceUtils.required(this.getById(id), "商家不存在");
        MerchantCopier.INSTANCE.copying(param, merchant);
        this.updateById(merchant);
    }

    @Override
    public Pagination<MerchantDTO> getMerchantPaging(@NonNull MerchantPagingParam param) {
        Page<Merchant> page = this.lambdaQuery()
                .eq(StringUtils.nonEmpty(param.getName()), Merchant::getName, param.getName())
                .eq(Objects.nonNull(param.getRecommend()), Merchant::getRecommend, param.getRecommend())
                .eq(Objects.nonNull(param.getStatus()), Merchant::getStatus, param.getStatus())
                .orderByDesc(Merchant::getCreateTime).page(PagingUtils.convert(param.getPaging()));
        return PagingUtils.convert(page, MerchantCopier.INSTANCE::copying);
    }
}
